package com.example.module.war;

import org.picketlink.idm.IdentityManager;
import org.picketlink.idm.PartitionManager;
import org.picketlink.idm.PermissionManager;
import org.picketlink.idm.credential.Password;
import org.picketlink.idm.model.basic.Realm;
import org.picketlink.idm.model.basic.User;
import org.picketlink.idm.permission.acl.spi.PersistentPermissionVoter;
import org.picketlink.idm.permission.spi.PermissionVoter;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.inject.Produces;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

@Singleton
@Startup
public class CreateUsers {
    private static final String JOHN_LOGIN_NAME = "john";
    private static final String JOHN_EMAIL = "john@doe.com";
    private static final String JOHN_FIRST_NAME = "John";
    private static final String JOHN_LAST_NAME = "Doe";
    private static final String JOHN_PASSWORD = "pass";
    private static final String JOHN_READ_PERMISSION_FILENAME = "readable-for-john.txt";
    private static final String JOHN_READ_PERMISSION_OPERATION = "read";

    @Resource(lookup = "java:/picketlink/JpaBasedPartitionManager")
    private  PartitionManager partitionManager;

    @PersistenceContext(unitName = "idm-module")
    private EntityManager em;

    @PostConstruct
    @TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
    public void create() {

        IdentityManager identityManager = null;
        PermissionManager permissionManager = null;

        Realm defaultRealm = this.partitionManager.getPartition(Realm.class, "default");
        if (defaultRealm==null){
//            System.out.println("### ######### ########### #### Couldn't find default partition, creating default partition");
            defaultRealm = new Realm("default");
            this.partitionManager.add(defaultRealm);
        } else {
//            System.out.println("##### #### #### ###### Found default partition");
        }

        identityManager = partitionManager.createIdentityManager(defaultRealm);
        permissionManager = partitionManager.createPermissionManager(defaultRealm);

        User john = newUser(JOHN_LOGIN_NAME, JOHN_EMAIL, JOHN_FIRST_NAME, JOHN_LAST_NAME, identityManager);

        Password pass = new Password(JOHN_PASSWORD);
        identityManager.updateCredential(john, pass);

        permissionManager.grantPermission(john, JOHN_READ_PERMISSION_FILENAME, JOHN_READ_PERMISSION_OPERATION);

        // PRINTS SOMETHING -> PERMISSION SHOULD BE GRANTED
        System.out.println("&&& &&&& " + permissionManager.listPermissions(JOHN_READ_PERMISSION_FILENAME));

    }

    private User newUser(String loginName, String email, String firstName, String lastName, IdentityManager identityManager) {

        User user = new User(loginName);
        user.setEmail(email);
        user.setFirstName(firstName);
        user.setLastName(lastName);


        identityManager.add(user);
        return user;
    }

    @Produces
    @ApplicationScoped
    public PermissionVoter producePermissionVoter(PartitionManager partitionManager) {
        return new PersistentPermissionVoter(partitionManager);
    }


}
